#We use just-in-time importing here to improve load times
#Here are the imports:
#import plotly.express as px
#import matplotlib.pyplot as plt
import pandas as pd


def plotlyGrid(data: pd.DataFrame, x: str, y: str, output="Output", title=""):
    """
    Calls px.scatter_3d with data. Returns a plotly figure.

    :param data: pandas dataframe with cols x, y, and output. Color is optional
    :param x: xcol in df
    :param y: ycol in df
    :param output: zcol in df
    :param title: Title for graph
    """
    try:
        import plotly.express as px
    except:
        raise Exception("Plotly is required to use this graph. Install with `pip install plotly`")
    if "Color" in data:
        color = "Color"
        colormap = "identity"
    else:
        color = None
        colormap = None
    fig = px.scatter_3d(data, x, y, output, color=color, color_discrete_map=colormap, title=title)
    return fig

def plotlyAnimation(data, x, y, anim, output="Output", title=""):
    """
    Calls px.scatter_3d with data and animation frame. Returns a plotly figure.

    :param data: pandas dataframe with cols x, y, anim, and output. Color is optional
    :param x: xcol in df
    :param y: ycol in df
    :param anim: column for animation
    :param output: zcol in df
    :param title: Title for graph
    """
    try:
        import plotly.express as px
    except:
        raise Exception("Plotly is required to use this graph. Install with `pip install plotly`")
    if "Color" in data:
        color = "Color"
        colormap = "identity"
    else:
        color = None
        colormap = None

    fig = px.scatter_3d(data, x, y, output, animation_frame=anim, color=color, color_discrete_map=colormap,
                        title=title, range_z=[data[output].min(), data[output].max()])
    return fig

def matplotlibGrid(data, x, y, output="Output", title=""):
    """
    Calls ax.scatter with data. Returns a plt instance, a fig, and the ax.

    :param data: pandas dataframe with cols x, y, and output. Color is optional
    :param x: xcol in df
    :param y: ycol in df
    :param output: zcol in df
    :param title: Title for graph
    """
    try:
        import matplotlib.pyplot as plt
    except:
        raise Exception("Matplotlib is required to use this graph. Install with `pip install matplotlib`")
    fig = plt.figure()
    ax = fig.add_subplot(projection='3d')

    if "Color" in data:
        color = data["Color"]
    else:
        color = None

    ax.scatter(data[x], data[y], data[output], c=color)
    ax.set_xlabel(x)
    ax.set_ylabel(y)
    ax.set_zlabel(output)
    ax.set_title(title)

    return plt, fig, ax


import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2' #stops agressive error message printing
from tensorflow.keras.models import Model
from tensorflow.python.keras.layers import deserialize, serialize
from tensorflow.python.keras.saving import saving_utils

def unpack(model, training_config, weights):
    restored_model = deserialize(model)
    if training_config is not None:
        restored_model.compile(
            **saving_utils.compile_args_from_training_config(
                training_config
            )
        )
    restored_model.set_weights(weights)
    return restored_model

# Hotfix function
def make_keras_picklable():
    print("<KERAS MP> Pickling magic")
    def __reduce__(self):
        model_metadata = saving_utils.model_metadata(self)
        training_config = model_metadata.get("training_config", None)
        model = serialize(self)
        weights = self.get_weights()
        return unpack, (model, training_config, weights)

    cls = Model
    cls.__reduce__ = __reduce__


#THIS EXAMPLE IS BROKEN AND IS CURRENTLY BEING BUGFIXED
raise NotImplementedError("THIS CODE IS CURRENTLY BROKEN") #TODO - bugfix

from MLVisualizationTools import Analytics, Interfaces, Graphs, Colorizers
from MLVisualizationTools.backend import fileloader
import pandas as pd
import os
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2' #stops agressive error message printing
from tensorflow import keras

try:
    import plotly
except:
    raise ImportError("Plotly is required to run this demo. If you don't have plotly installed, install it with"
                      " `pip install plotly' or run the matplotlib demo instead.")

def main():
    model = keras.models.load_model(fileloader('examples/Models/titanicmodel'))
    df: pd.DataFrame = pd.read_csv(fileloader('examples/Datasets/Titanic/train.csv'))

    AR = Analytics.Tensorflow(model, df, ["Survived"])
    maxvar = AR.maxVariance()
    #
    # grid = Interfaces.TensorflowAnimation(model, maxvar[0].name, maxvar[1].name, maxvar[2].name,
    #                                  df, ["Survived"])
    # grid = Colorizers.Binary(grid, highcontrast=True)
    # print(grid)
    # fig = Graphs.PlotlyAnimation(grid, maxvar[0].name, maxvar[1].name, maxvar[2].name)
    # fig.show()
    # with open('plotly.html', 'w+') as f:
    #     f.write(fig.to_html())

    grid = Interfaces.TensorflowAnimation(model, 'Parch', 'SibSp', maxvar[0].name,
                                          df, ["Survived"], steps=5)
    print(grid.head(125))
    grid = Colorizers.Binary(grid, highcontrast=True)
    fig = Graphs.PlotlyAnimation(grid, 'Parch', 'SibSp', maxvar[0].name)
    fig.show()

if __name__ == "__main__":
    main()